<?php
/*
Eenos - Server Provisioning Module
@copyright Copyright (c) eenos.com
@license https://www.gnu.org/licenses/gpl-3.0.txt, GNU GENERAL PUBLIC LICENSE Version 3
*/

use WHMCS\Database\Capsule;
use WHMCS\Auth;
use WHMCS\Service\Status;
use WHMCS\Module\Server\CustomAction;
use WHMCS\Module\Server\CustomActionCollection;
if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}
// Enable Debug logs true , false
define("EENOSLOG", false);
global $eenos_conf;

// Load Eenos Control Panel Pors
$eenos_conf['panel_ports']['wap_ssl'] = '5555';
$eenos_conf['panel_ports']['wap_nonssl'] = '5554';
$eenos_conf['panel_ports']['rapp_ssl'] = '4444';
$eenos_conf['panel_ports']['rapp_nonssl'] = '4443';
$eenos_conf['panel_ports']['uapp_ssl'] = '3333';
$eenos_conf['panel_ports']['uapp_nonssl'] = '3332';

if(!empty($eenos_conf['custom_ports']['wap_ssl'])){
	$eenos_conf['panel_ports']['wap_ssl'] = $eenos_conf['custom_ports']['wap_ssl'];
}

if(!empty($eenos_conf['custom_ports']['wap_nonssl'])){
	$eenos_conf['panel_ports']['wap_nonssl'] = $eenos_conf['custom_ports']['wap_nonssl'];
}
// For future options
if(!empty($eenos_conf['custom_ports']['rapp_ssl'])){
	$eenos_conf['panel_ports']['rapp_ssl'] = $eenos_conf['custom_ports']['rapp_ssl'];
}
if(!empty($eenos_conf['custom_ports']['rapp_nonssl'])){
	$eenos_conf['panel_ports']['rapp_nonssl'] = $eenos_conf['custom_ports']['rapp_nonssl'];
}
if(!empty($eenos_conf['custom_ports']['uapp_ssl'])){
	$eenos_conf['panel_ports']['uapp_ssl'] = $eenos_conf['custom_ports']['uapp_ssl'];
}
if(!empty($eenos_conf['custom_ports']['uapp_nonssl'])){
	$eenos_conf['panel_ports']['uapp_nonssl'] = $eenos_conf['custom_ports']['uapp_nonssl'];
}
// End future options
// A simple log module  for debug
function eenos_debug_log($data){
    if (EENOSLOG) {
        $logfile=dirname(__FILE__).'/debug.log';
        if(is_array($data)) {
            file_put_contents($logfile,  print_r($data, true), FILE_APPEND);
        
        } else {
            file_put_contents($logfile, $data . "\n", FILE_APPEND);
        }
    }
     
}

// Eenos whmcs module meta data
function eenos_MetaData()
{
    return array(
        'DisplayName' => 'Eenos',
        'APIVersion' => '1.1', // Use API Version 1.1
        'RequiresServer' => true, // Set true if module requires a server to work
        'DefaultNonSSLPort' => $GLOBALS['eenos_conf']['panel_ports']['wap_nonssl'], // Default Non-SSL Connection Port
        'DefaultSSLPort' => $GLOBALS['eenos_conf']['panel_ports']['wap_ssl'], // Default SSL Connection Port        
        'ServiceSingleSignOnLabel' => 'Login to Eenos as User',
        'AdminSingleSignOnLabel' => 'Eenos WAP Login',
        'ListAccountsProductField' => 'configoption1',
        'ListAccountsUniqueIdentifierDisplayName' => 'Domain',
        'ListAccountsUniqueIdentifierField' => 'domain',
    );
}

function get_eenos_serverip($params){
    // We use host name to match ssl reequests
    return (!empty($params["serverhostname"]) ? $params["serverhostname"] : $params["serverip"]);
    // If you like to use IP  instead of hostname
	// return (!empty($params["serverip"]) ? $params["serverip"] : $params["serverhostname"]);
}
// Do Api actions
function eenos_PerformOperation(array $apidata,$action_url)
{    
    eenos_debug_log($apidata);
    eenos_debug_log($action_url);
	$statusCodes =array(
        "200" => "OK",
        "426" => "Your package limit exceeded",
        "208" => "License already exist",
        "400" => "Bad request, send invalid data to API",
        "204" => "Internal error, you need to contact support",
        "304" => "License Terminated,unable to suspend",
        "202" => "Package limit exceeded. you need to remove some licenses to downgrade the package",
        "404" => "License package not found",
        "204" => "Internal error, you need to contact support",
        "403" => "Firbidden",
        "207" => "You can only reissue a terminated license",
        "500" => "Server Error",
        "700" => "php-curl is missing in this whmcs server",
    );	
	$verifyurl=$action_url;
	if (function_exists("curl_exec")) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $verifyurl);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $apidata);
            curl_setopt($ch, CURLOPT_TIMEOUT, 300);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            // Turn off the server and peer verification (TrustManager Concept).
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
            // UserAgent
		    curl_setopt($ch, CURLOPT_USERAGENT, 'Eenos');
            $data = curl_exec($ch);
            if (curl_error($ch)) {
                throw new Exception('Unable to connect: ' . curl_errno($ch) . ' - ' . curl_error($ch));
            } elseif (empty($data)) {
                throw new Exception('Empty response');
            }
            curl_close($ch);
            eenos_debug_log("Api raw output ...");
            eenos_debug_log($data);
            $remoteresult=$data;
    }else{
            $remoteresult="{\"status\":\"500\",\"message\":\"php-curl is missing in this whmcs server\"}";
    }
	$output=json_decode($remoteresult,true);        
    if (is_null($output)) {
        throw new Exception('Invalid response format');
    }    

    if (isset($output['status'])){
        $result['success']=$output["status"];
       
    }else{
        if(isset($output["username"])){
            $output["status"]="200";
        }else{
            $output["status"]="500" ;           
        }
    }
    if (isset($output["data"])){        
        $result['message']=$output['data'];
       
    }else{        
        $result['message']=$output['info'];
    }
    $result['data']=$output;       
    return($result);

}
function get_plans($server_data){
    // api action for getting packages from a server
    eenos_debug_log("----- server ----");
    eenos_debug_log($server_data);
    //  We use hostname instead of ip to give more ssl security
    $server=!empty($server_data["hostname"]) ? $server_data["hostname"] : $server_data["ipaddress"];
    // $server=!empty($server_data["ipaddress"]) ? $server_data["ipaddress"] : $server_data["hostname"];    
    eenos_debug_log($server);
    // Find protocol and port for API connection
    if ($server_data['secure'] == "on"){
        $proto='https';
        if(empty($server_data['port'])){
            $port=$GLOBALS['eenos_conf']['panel_ports']['wap_ssl'];
        }else{
            $port=$server_data['port'];
        }
    }else{
        $proto='http';
        if(empty($server_data['port'])){
            $port=$GLOBALS['eenos_conf']['panel_ports']['wap_nonssl'];
        }else{
            $port=$server_data['port'];
        }
    }
    $apiurl='/listpackages/';
    $action_url = $proto.'://'.$server.':'.$port.'/api'.$apiurl;
    $apidata['apiuser']=$server_data['username'];
    $apidata['apiauth']=eenos_server_pass_from_whmcs($server_data['password']);    
    $result=eenos_PerformOperation($apidata,$action_url);  
    eenos_debug_log($result);      
    if($result['success']=="200"){
        if(!empty($result['data']['data']['packages'])){
            $pkgs=explode(',',$result['data']['data']['packages']);
        }else{
            $pkgs[0]='Default';
        }
    }
    sort($pkgs);   
    return ($pkgs);
    
}

function get_api_logins($params){
    // global $eenos_conf;
    $apidata=array();    
    if(!empty($GLOBALS['eenos_conf']['panel_ports']['wap_ssl'])){
        $port=$GLOBALS['eenos_conf']['panel_ports']['wap_ssl'];
        // $proto='https';
    }else{
        $port=$GLOBALS['eenos_conf']['panel_ports']['wap_nonssl'];
        // $proto='http';
    }
    $proto=$params["serverhttpprefix"];
    $server=get_eenos_serverip($params);   
    $apidata['server']=$server;
    $apidata['apiuser']=$params["serverusername"];
    $apidata['apiauth']=$params["serverpassword"];
    $apidata['base_url']=$proto.'://'.$server.':'.$port.'/api';    
    return ($apidata);
}

// Eenos Whmcs Configurable Options.
function eenos_ConfigOptions()
{  
    /*
    if(!empty($_REQUEST['id'])){		
		$hostpkgs = Capsule::table('tblcustomfields')
			->where('type', 'product')
			->where('relid', $_REQUEST['id'])
			->where('fieldname','like','EenosHostingPackage|%')
			->first();
		      
		if(empty($hostpkgs)){
            eenos_debug_log("Auto custom filed.");
			$ret_uuid = Capsule::table('tblcustomfields')->insert(array(
				array('type' => 'product',
					'relid' => $_REQUEST['id'], 
					"fieldname" => "EenosHostingPackage|Hosting Package", 
					"fieldtype" => "text", 
					"adminonly" => "on"
				)
			));
		}		
	}
    */
    // Get the product ID
	$pid = (int) $_REQUEST['id'];
    // Get the list of servers in this Server Group
    $res = Capsule::table('tblproducts')->where('id', $pid)->get();	
	$product = (array) $res[0];
    // eenos_debug_log($product);
	if(!empty($product['servergroup'])){
		$res = Capsule::table('tblservergroupsrel')->where('groupid', $product['servergroup'])->get();
        eenos_debug_log($res);
		foreach($res as $re){
			$server_list[$re->serverid] = '';
		}
	}
    // eenos_debug_log($server_list);
    // Get Eenos Servers
	if(!empty($server_list)){		
		$result = Capsule::table('tblservers')
			->where('type', 'eenos')
			->whereIn('id', array_keys($server_list))
			->get();
		
		if(empty($result)){
			echo '<font color="#FF0000">The Eenos servers could not be found. Please add the Eenos Server and Server group to proceed</font>';
			return;
		}
		
		$server_list = array();		
		foreach($result as $re){
			$server_list[$re->id] = $re->id.' - '.trim($re->name);
			$server_data[$re->id] = (array) $re;
		}
	
	}
	
	if(!empty($server_list)){
		$i = 0;
		foreach($server_list as $sk => $server_name){			
			if($i >= 5){
				break;
			}			
			$sv = $server_data[$sk];			
			$_hostname = $sv['hostname'];
			if(empty($_hostname)){
				$_hostname = $sv['ipaddress'];
			}
			// Get the packages data from Eenos
			eenos_debug_log($server_data);
            eenos_debug_log($_hostname);
			$data = get_plans($sv);			
			// Build the options list to show Plans
			if(!empty($data)){
				foreach($data as $k){
					$eenos_plans[$k] = $k.' - ['.$_hostname.']';
				}
			}
		}
	}
    if(!empty($eenos_plans)){
        #configoption1
        $configarray['EenosPackageName'] = array("FriendlyName" => "Eenos Package Name", "Type" => "dropdown", "Options" =>$eenos_plans );
    }else{  
        #configoption1  
        $configarray['EenosPackageName'] = array("FriendlyName" => "Eenos Package Name","Type" => "text", "Size" => 25,"Default" => "Default",);	
    }
    #configoption2
    $configarray['IPv4'] = array("FriendlyName" => "Dedicated IPv4","Type" => "yesno","Description" => "Tick to assign dedicated IPv4 if available",); #configoption2
    $configarray['DiskQuota'] = array("FriendlyName" => "Web Space Quota <strong>(MB)</strong>","Type" => "text", "Size" => 10,"Description" => "( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption3
    $configarray['IPv6'] = array("FriendlyName" => "Dedicated IPv6","Type" => "yesno","Description" => "Tick to assign dedicated IPv6 if available", ); #configoption4
    $configarray['BandWidth'] = array("FriendlyName" => "Bandwidth Limit <strong>(MB)</strong>","Type" => "text", "Size" => 10,"Description" => "( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption5
    $configarray['DKIM'] = array("FriendlyName" => "DKIM","Type" => "dropdown","Options" => ["on" => "On","off"=>"Off",],"Description" => "Domain Keys Identified Mail","Default" => "on", ); #configoption6
    $configarray['FtpAccounts'] = array("FriendlyName" => "Max FTP Accounts","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption7
    $configarray['SPF'] = array("FriendlyName" => "SPF","Type" => "dropdown","Options" => ["on" => "On","off"=>"Off",],"Description" => "Sender Policy Framework","Default" => "on", ); #configoption8
    $configarray['EmailAccounts'] = array("FriendlyName" => "Max Email Accounts","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption9
    $configarray['UseRegistrarNS'] = array("FriendlyName" => "Use Domain Registrar NS","Type" => "dropdown","Options" => ["off"=>"Off","on" => "On",],"Description" => "Use domain registry NS records for creating domain","Default" => "off", ); #configoption10
    $configarray['AddonDomain'] = array("FriendlyName" => "Max Addon Domains","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption11
    $configarray['ShellAccess'] = array("FriendlyName" => "Shell Access","Type" => "dropdown","Options" => ["off" => "Off","on"=>"On",],"Description" => "Enable Shell access","Default" => "off", ); #configoption12
    $configarray['SubDomain'] = array("FriendlyName" => "Max Subdomains","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption13
    $configarray['MailExchange'] = array("FriendlyName" => "Mail Exchange Server","Type" => "dropdown","Options" => ["local" => "Local","remote"=>"Remote",],"Description" => "The SMTP server of the related domain.","Default" => "local", ); #configoption14
    $configarray['ParkedDomain'] = array("FriendlyName" => "Max Parked domains","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption15
    $configarray['Databases'] = array("FriendlyName" => "Max Databases","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption16   
    $configarray['EmailQuota'] = array("FriendlyName" => "Default Email Account Quota","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption17
    $configarray['Theme'] = array("FriendlyName" => "Eenos Theme","Type" => "text", "Size" => 15,"Default" => "default",); #configoption18
    if($product['type']=="reselleraccount"){
        $configarray['SelfOwn'] = array("FriendlyName" => "<strong>Reseller Ownership</strong> <br /><small class='text-danger'>Danger to enable this option</small>","Type" => "yesno","Description" => "Set the reseller to own there own account.",); #configoption19
        $configarray['LimitReseller'] = array("FriendlyName" => "<strong>Limit Reseller by Number</strong> <br /><small>Enter max number of allowed accounts</small>","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption20
        $configarray['ResellerQuota'] = array("FriendlyName" => "<strong>Reseller Disk Quota (MB)</strong> <br /><small>Limit disk space usage of all sub accounts</small>","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption21
        $configarray['ResellerBandwidth'] = array("FriendlyName" => "<strong>Reseller Bandwidth (MB)</strong> <br /><small>Limit bandwidth usage of all sub accounts</small>","Type" => "text", "Size" => 10,"Description" => " ( Enter <strong>unlimited</strong> for Unlimited use )",); #configoption22
    }
    eenos_debug_log("--------Module configurationa ---------------");
    eenos_debug_log($product);
    // eenos_debug_log($configarray);
    return $configarray;
    /*	
	if(empty($eenos_plans)){

		return $configarray;
	}
	eenos_debug_log($eenos_plans);
	$configarray['Select Hosting Plan'] = array("Type" => "dropdown", "Options" =>$eenos_plans );	
	return $configarray;
    */
}

function eenos_GetRemoteMetaData($params){
	
	try {
        eenos_debug_log($params);
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);           
        $post_url='/load-average/';
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;
        $post_data['apiuser']=$params["serverusername"];
        $post_data['apiauth']=$params["serverpassword"];                      
        $loadresult=eenos_PerformOperation($post_data,$action_url);
        eenos_debug_log($loadresult);
        if(!empty($loadresult['data']['data']['load_average'])){           
            $rawload=explode(" ",$loadresult['data']['data']['load_average']);            
            $mdata['load']=$rawload;
        }else{
            $mdata['load']="N/A";
        }
        $post2_url='/version/';
        $action2_url=$proto.'://'.$server.':'.$port.'/api'.$post2_url;
        $versionresult=eenos_PerformOperation($post_data,$action2_url);        
        eenos_debug_log($versionresult);
        if(!empty($versionresult['data']['data']['version'])){   
            $mdata['version']=$versionresult['data']['data']['version'];
        }else{
            $mdata['version']="Unknown";
        }
        $post3_url='/listpackages/';
        $action3_url=$proto.'://'.$server.':'.$port.'/api'.$post3_url;
        $pkgresult=eenos_PerformOperation($post_data,$action3_url); 
        eenos_debug_log($pkgresult);
        if(!empty($pkgresult['data']['data']['packages'])){   
            $mdata['packages']=$pkgresult['data']['data']['packages'];
        }else{
            $mdata['packages']="Default";
        }		
        eenos_debug_log($mdata);
        return $mdata;
	} catch (Exception $e) {
		return array('success' => false, 'error' => 'Could not connect');
	}
}

function eenos_RenderRemoteMetaData($params){	
	try{		
		$remoteData = $params['remoteData'];
        eenos_debug_log( $params['remoteData']);
		if($remoteData){
			$metaData = $remoteData->metaData;
		}else{
		    $metaData['version']='Unknown';
            $metaData['load']="Unknown";
        }
		if(!empty($metaData)){
		
			if(is_array($metaData['load'])){
				foreach($metaData['load'] as $k => $v){
					$metaData['load'][$k] = round($v, 2);
				}
                $metaData['load']=implode(', ', $metaData['load']);               
			}
            if (!empty($metaData['packages'])){
                $packages=$metaData['packages'];
            }else{
                $packages="Default";
            }
            $pkgs=explode(",",$packages);
            sort($pkgs);
            // eenos_debug_log($pkgs);	
            $pdata='';
            foreach($pkgs as $pname){
                $pdata.="<option value=\"$pname\">$pname</option>\n";
            }
            $fpkgs="<select name=\"packages\">\n$pdata\n</select>";
			return ("Eenos Version: <span style='color: #ff8b00;'>" . $metaData['version'] . "</span><br>\nLoad Average: " .$metaData['load']."<br />Hosting Packages: ".$fpkgs);
		}
	
	}catch(Exception $e){
		 // Record the error in WHMCS's module log.
         logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
	}
	
}

/**
 * Test Eenos API connection with the given server parameters.
 */
function eenos_TestConnection(array $params)
{
    // file_put_contents(dirname(__FILE__).'/log.txt', print_r($params, true)); 
    try {    
        eenos_debug_log("Eenos test connection ..");
        eenos_debug_log($params);
        // $apidata=get_api_logins($params);   
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);   
        $apidata['server']=$server;
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['base_url']=$proto.'://'.$server.':'.$port.'/api';    
        $post_url='/test_connection/';
        $post_data['apiuser']=$apidata['apiuser'];
        $post_data['apiauth']=$apidata['apiauth'];
        $action_url=$apidata['base_url'].$post_url;            
        $result=eenos_PerformOperation($post_data,$action_url);
        $success = true;
        $errorMsg = '';
        // error_log(print_r($post_data, TRUE));
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }   
    if($result['success']=="200"){
        return array('success' => true);
    }else{
        return($result['message']);
    }
}


/**
 * Perform  wap single sign-on for a server.
 */
function eenos_AdminSingleSignOn(array $params)
{
    try {
        eenos_debug_log($params);
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);           
        $post_url='/wapsso/';
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;
        $post_data['apiuser']=$params["serverusername"];
        $post_data['apiauth']=$params["serverpassword"];
        $post_data['user']='root';                   
        $result=eenos_PerformOperation($post_data,$action_url);
        eenos_debug_log($result);
        $response = array();
        if(!empty($result['data']['data']['url'])){
            $success = true;
            $response['redirectUrl']=$result['data']['data']['url'];
            $errorMsg = '';
            eenos_debug_log($response);
            return array(
                'success' => true,               
                'redirectTo' => $response['redirectUrl'],
            );
        }else{
            $success = false;
            $response['redirectUrl']='#';
            $errorMsg = $result['message'];
            return array(
                'success' => false,
                'errorMsg' => $errorMsg,
            );
        }      
      
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}
/*
Perfor User or Reseller Single Sign-On
*/
function eenos_ServiceSingleSignOn(array $params)
{
    try {
        // eenos_debug_log($params);
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);
        if( $params["producttype"] == "reselleraccount"){
            $post_url='/rappsso/';
        }else{
            $post_url='/uappsso/';
        }
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;
        $post_data['apiuser']=$params["serverusername"];
        $post_data['apiauth']=$params["serverpassword"];
        $post_data['user']= $params["username"];
        // eenos_debug_log( $post_data) ;
        // eenos_debug_log($action_url);
        $result=eenos_PerformOperation($post_data,$action_url);
        eenos_debug_log($result);
        $response = array();
        if(!empty($result['data']['data']['url'])){
            $success = true;
            $response['redirectUrl']=$result['data']['data']['url'];
            $errorMsg = '';
            eenos_debug_log($response);
            return array(
                'success' => true,               
                'redirectTo' => $response['redirectUrl'],
            );
        }else{
            $success = false;
            $response['redirectUrl']='#';
            $errorMsg = $result['message'];
            return array(
                'success' => false,
                'errorMsg' => $errorMsg,
            );
        }              
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}

function eenos_GetUserCount($params){
	
	try{
        eenos_debug_log($params);
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);           
        $post_url='/get_current_users_count/';
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;
        $post_data['apiuser']=$params["serverusername"];
        $post_data['apiauth']=$params["serverpassword"];
        $post_data['user']='root';                   
        $result=eenos_PerformOperation($post_data,$action_url);
        eenos_debug_log($result);
		$total_users = $owned_users = 0;
		
		if(!empty($result['data']['data']['users'])){
			// The API call is made via the serverusername so it will return only owned users
			$total_users = $owned_users = $result['data']['data']['users'];
		}
		
		return array('success' => true, 'totalAccounts' => $total_users, 'ownedAccounts' => $owned_users);
		
	}catch(Exception $e){
		return array('success' => false, 'error' => 'could not connect');
	}
}


/*
Provision a new Eenos Hosting Account on server 

*/
function  eenos_save_user_data($params,$hostingdata,$doupdate=False){
    $serviceid = $params["serviceid"];
    if(!empty($hostingdata['dedicatedip']['ipv4'])){
        $ips['ipv4']=$hostingdata['dedicatedip']['ipv4'];
        if(!empty($hostingdata['dedicatedip']['status']) && $hostingdata['dedicatedip']['status'] == "on"){
            Capsule::table('tblhosting')
			->where('id',$serviceid)
			->update(
				array('dedicatedip' => $ips['ipv4'])
			);
        }
    }
    if(!empty($hostingdata['dedicatedip']['ipv6'])){
        $ips['ipv6']=$hostingdata['dedicatedip']['ipv6'];
    }
    $assignedips='';
    foreach($ips as $key => $val){
        $iparr[]=$val;
    }
    $assignedips=implode(",", $iparr);
    Capsule::table('tblhosting')
        ->where('id', $serviceid)
        ->update(
            array('assignedips' => $assignedips)
        );
    if ($params['producttype']== "reselleraccount"){
        if(!empty($hostingdata['reseller_data']['reseller_bandwidth'])){
            if($hostingdata['reseller_data']['reseller_bandwidth']=="unlimited"){
                $bwlimit=0;
            }else{
                $bwlimit=(int)$hostingdata['reseller_data']['reseller_bandwidth'];
            }
        }else{
            $bwlimit=0;
        }
        if(!empty($hostingdata['reseller_data']['reseller_quota'])){
            if($hostingdata['reseller_data']['reseller_quota']=="unlimited"){
                $disklimit=0;
            }else{
                $disklimit=(int)$hostingdata['reseller_data']['reseller_quota'];
            }
        }else{
            $disklimit=0;
        }
    }else{
        if($hostingdata['bandwidth']=="unlimited"){
            $bwlimit=0;
        }else{
            $bwlimit=(int)$hostingdata['bandwidth'];
        }
        if($hostingdata['quota']=="unlimited"){
            $disklimit=0;
        }else{
            $disklimit=(int)$hostingdata['quota'];
        }
    }
    $created_at=$hostingdata['setuptime'];
    if(!empty($hostingdata['nameservers'][0])){
        $ns1=$hostingdata['nameservers'][0];
    }else{
        $ns1='';        
    }
    if(!empty($hostingdata['nameservers'][1])){
        $ns2=$hostingdata['nameservers'][1];
    }else{
        $ns2='';        
    }
    Capsule::table('tblhosting')
        ->where('id', $serviceid)        
        ->update([            
            'disklimit' => $disklimit,
            'ns1' => $ns1,
            'ns2' => $ns2,
            'bwlimit' => $bwlimit,
            'created_at' => $created_at,
            'lastupdate' => Capsule::raw('now()'),
                ]);
    if ($doupdate){        
        eenos_debug_log("Running account data sync");  
        if ($params['producttype']== "reselleraccount"){
            if(!empty($hostingdata['quota_usage_reseller'])){
                $diskusage=(int)$hostingdata['quota_usage_reseller'];
            }else{
                $diskusage=0;
            }
            if(!empty($hostingdata['bandwidth_usage_reseller'])){
                $bwusage=(int)$hostingdata['bandwidth_usage_reseller'];
            }else{
                $bwusage=0;
            }
        }else{
            if(!empty($hostingdata['quota_usage'])){
                $diskusage=(int)$hostingdata['quota_usage'];
            }else{
                $diskusage=0;
            }
            if(!empty($hostingdata['bandwidth_usage'])){
                $bwusage=(int)$hostingdata['bandwidth_usage'];
            }else{
                $bwusage=0;
            }
        }
        Capsule::table('tblhosting')
            ->where('id', $serviceid)        
            ->update([            
                'diskusage' => $diskusage,
                'bwusage' => $bwusage,
                ]);
    }
    // eenos_debug_log($params);

}
/*
Create hosting account action
*/
function eenos_CreateAccount(array $params)
{
    try {        
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"];       
        $apidata['password']=$params["password"];
        $apidata['domain']=$params["domain"];
        $apidata['email']= $params["clientsdetails"]["email"];
        $apidata['package']=!empty($params['configoption1']) ? $params['configoption1'] : 'Default';
        if((!empty($params['configoption2']))&& ($params['configoption2']=="on")) {
            // Decicated ipv4 
            $apidata['ipv4']="on";
        }
        if(!empty($params['configoption3'])) {
            // Web space quota
            if(is_numeric($params['configoption3'])){
                $apidata['quota']=$params['configoption3'];
            }else{
                $apidata['quota']='unlimited';
            }            
        }
        if((!empty($params['configoption4']))&& ($params['configoption4']=="on")) {
            // Decicated ipv6 
            $apidata['ipv6']="on";
        }
        if(!empty($params['configoption5'])) {
            // bandwidth
            if(is_numeric($params['configoption5'])){
                $apidata['bandwidth']=$params['configoption5'];
            }else{
                $apidata['bandwidth']='unlimited';
            }            
        }
        if((!empty($params['configoption6']))&& ($params['configoption6']=="off")) {
            // DKIM
            $apidata['dkim']="off";
        }else{
            $apidata['dkim']="on";
        }
        if(!empty($params['configoption7'])) {
            // Ftp accounts
            if(is_numeric($params['configoption7'])){
                $apidata['ftp']=$params['configoption7'];
            }else{
                $apidata['ftp']='unlimited';
            }            
        }
        if((!empty($params['configoption8']))&& ($params['configoption8']=="off")) {
            // SPF
            $apidata['spf']="off";
        }else{
            $apidata['spf']="on";
        }
        if(!empty($params['configoption9'])) {
            // Email accounts
            if(is_numeric($params['configoption9'])){
                $apidata['email_accounts']=$params['configoption9'];
            }else{
                $apidata['email_accounts']='unlimited';
            }            
        }
        if(!empty($params['configoption10'])) {
            // USe dns registrar ns
            if($params['configoption10']=="on" ){
                $apidata['use_registrar_ns']="on";
            }else{
                $apidata['use_registrar_ns']="off";
            }         
        }
        if(!empty($params['configoption11'])) {
            // Addon domains
            if(is_numeric($params['configoption11'])){
                $apidata['addondomains']=$params['configoption11'];
            }else{
                $apidata['addondomains']='unlimited';
            }            
        }
        if(!empty($params['configoption12'])) {
            // shell acceess
            if($params['configoption12']=="on" ){
                $apidata['shell']="on";
            }else{
                $apidata['shell']="off";
            }         
        }
        if(!empty($params['configoption13'])) {
            //sub domain
            if(is_numeric($params['configoption13'])){
                $apidata['subdomains']=$params['configoption13'];
            }else{
                $apidata['subdomains']='unlimited';
            }            
        }
        if(!empty($params['configoption14'])) {
            if($params['configoption14']=="remote" ){
                $apidata['mailserver']="remote";
            }else{
                $apidata['mailserver']="local";
            }         
        }else{
            $apidata['mailserver']="local";
        }
        if(!empty($params['configoption15'])) {
            // parked domains
            if(is_numeric($params['configoption15'])){
                $apidata['parkeddomains']=$params['configoption15'];
            }else{
                $apidata['parkeddomains']='unlimited';
            }            
        }
        if(!empty($params['configoption16'])) {
            // sql
            if(is_numeric($params['configoption16'])){
                $apidata['sql']=$params['configoption16'];
            }else{
                $apidata['sql']='unlimited';
            }            
        }
        if(!empty($params['configoption17'])) {
            // Email default quota
            if(is_numeric($params['configoption17'])){
                $apidata['mailquota']=$params['configoption17'];
            }else{
                $apidata['mailquota']='unlimited';
            }            
        }
        if(!empty($params['configoption18'])) {
            // Eenos theme
            if($params['configoption18']=="default" ){
                $apidata['theme']="default";
            }else{
                $apidata['theme']=$params['configoption18'];
            }         
        }
        if( $params["producttype"] == "reselleraccount"){
            // Enable Reseller
            $apidata['reseller']="on";
            if(!empty($params['configoption19'])) {
                // Self own reseller 
                if($params['configoption19']=="on" ){
                    $apidata['resellerself']="on";
                }else{
                    $apidata['resellerself']="off";
                }
            }else{
                    $apidata['resellerself']="off";            
            }
            if(!empty($params['configoption20'])) {
                // Limit reseller
                if(is_numeric($params['configoption20'])){
                    $apidata['reseller_limt']=$params['configoption20'];
                }else{
                    $apidata['reseller_limt']='unlimited';
                }            
            }else{
                $apidata['reseller_limt']='unlimited';
            }
            if(!empty($params['configoption21'])) {
                // Email default quota
                if(is_numeric($params['configoption21'])){
                    $apidata['reseller_quota']=$params['configoption21'];
                }else{
                    $apidata['reseller_quota']='unlimited';
                }            
            }else{
                $apidata['reseller_quota']='unlimited';
            }
            if(!empty($params['configoption22'])) {
                // Email default quota
                if(is_numeric($params['configoption22'])){
                    $apidata['reseller_bandwidth']=$params['configoption22'];
                }else{
                    $apidata['reseller_bandwidth']='unlimited';
                }            
            }else{
                $apidata['reseller_bandwidth']='unlimited';
            }
        }
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);           
        $post_url='/createacct/';
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);
        eenos_debug_log($result);        
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return $e->getMessage();
    }
    if($result['success']=="200"){
        $hostingdata=$result['message'];
        eenos_save_user_data($params,$hostingdata);
        return 'success';
    }else{
        return($result['message']);
    }    
}

/*
Suspend Eenos Hosting Account
 */
function eenos_SuspendAccount(array $params)
{
    try {
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"];              
        if( $params["producttype"] == "reselleraccount"){
            $post_url='/suspendreseller/';           
        }else{
            $post_url='/suspendacct/';   
        }
        if(!empty($params["suspendreason"])){
            $apidata['reason']=$params["suspendreason"];
        }else{
            $apidata['reason']="Suspended from WHMCS";
        }
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);             
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    } 
}

/*
Un Suspend Eenos hosting accounts and resellers
*/
function eenos_UnsuspendAccount(array $params)
{
    try {
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"];              
        if( $params["producttype"] == "reselleraccount"){
            $post_url='/unsuspendreseller/';           
        }else{
            $post_url='/unsuspendacct/';   
        }        
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);         
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    } 
}

/**
 * Terminate Eenos Hosting account or reseller
 */
function eenos_TerminateAccount(array $params)
{
    try {
        $serviceid = $params["serviceid"];
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"];              
        if( $params["producttype"] == "reselleraccount"){
            $post_url='/terminatereseller/';           
        }else{
            $post_url='/removeacct/';   
        }        
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result); 
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){        
        Capsule::table('tblhosting')
            ->where('id', $serviceid)        
            ->update([            
                'disklimit' => 0,
                'ns1' => '',
                'ns2' => '',
                'bwlimit' => '',
                'bwusage' => 0,
                'diskusage' => 0,
                'dedicatedip' => '',
                'assignedips' => '',                
                'created_at' => '0000-00-00 00:00:00',
                'lastupdate' => Capsule::raw('now()'),
                ]);        
        return 'success';
    }else{
        return($result['message']);
    } 
}

/*
 * Change the password for Eenos hosting and reseller accounts
*/
function eenos_ChangePassword(array $params)
{
    try {
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"]; 
        $apidata['password']=$params["password"];             
        $post_url='/passwd/';          
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);         
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    }    
}

/**
 * Upgrade or downgrade an instance of a product/service.
 *
 */
function eenos_ChangePackage(array $params)
{
    try {        
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"]; 
        $apidata['package']=$params['configoption1'];                   
        $post_url='/changepackage/';          
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);     
        
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    }
}

/**
 * Renew  eenos hosting account. *
 * Attempt to renew an existing instance of a given product/service. This is
 * called any time a product/service invoice has been paid. 
 */
function eenos_Renew(array $params)
{
    $status=$params['status'];
    if($status == "Active"){
        return 'success';
    }else{
        try {        
            $apidata['apiuser']=$params["serverusername"];
            $apidata['apiauth']=$params["serverpassword"];
            $apidata['apiowner']="root";
            $apidata['user']=$params["username"];              
            if( $params["producttype"] == "reselleraccount"){
                $post_url='/unsuspendreseller/';           
            }else{
                $post_url='/unsuspendacct/';   
            }        
            $proto=$params["serverhttpprefix"];
            $port=$params["serverport"];
            $server=get_eenos_serverip($params);                   
            $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
            $result=eenos_PerformOperation($apidata,$action_url);
            eenos_debug_log($apidata);        
            eenos_debug_log($result);            
        } catch (Exception $e) {
            // Record the error in WHMCS's module log.
            logModuleCall(
                'eenos',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
            );

            return $e->getMessage();
        }
        if($result['success']=="200"){
            Capsule::table('tblhosting')
                ->where('id',$params['serviceid'] )            
                ->update(['domainstatus' => "Active"]);
            return 'success';
        }else{
            $error="Auto Renew Failed : This account is already active or doesn't exists in the server.";
            return($error);
        }  
    }  
}

// Account usage sync update function
function  eenos_save_usage_data($params,$user,$hostingdata){    
    $server_id=$params['serverid'];
    if($hostingdata['is_reseller']=="yes"){
        $producttype="reselleraccount";
    }else{
        $producttype="hostingaccount";
    }
    if(!empty($hostingdata['dedicatedip']['ipv4'])){
        $ips['ipv4']=$hostingdata['dedicatedip']['ipv4'];
        if(!empty($hostingdata['dedicatedip']['status']) && $hostingdata['dedicatedip']['status'] == "on"){
            Capsule::table('tblhosting')
            ->where('server', $params['serverid'])
			->where('username',$user)
			->update(
				array('dedicatedip' => $ips['ipv4'])
			);
        }
    }
    if(!empty($hostingdata['dedicatedip']['ipv6'])){
        $ips['ipv6']=$hostingdata['dedicatedip']['ipv6'];
    }
    $assignedips='';
    foreach($ips as $key => $val){
        $iparr[]=$val;
    }
    $assignedips=implode(",", $iparr);
    Capsule::table('tblhosting')
        ->where('server', $params['serverid'])
        ->where('username',$user)        
        ->update(
            array('assignedips' => $assignedips)
        );
    if ($producttype == "reselleraccount"){
        if(!empty($hostingdata['reseller_data']['reseller_bandwidth'])){
            if($hostingdata['reseller_data']['reseller_bandwidth']=="unlimited"){
                $bwlimit=0;
            }else{
                $bwlimit=(int)$hostingdata['reseller_data']['reseller_bandwidth'];
            }
        }else{
            $bwlimit=0;
        }
        if(!empty($hostingdata['reseller_data']['reseller_quota'])){
            if($hostingdata['reseller_data']['reseller_quota']=="unlimited"){
                $disklimit=0;
            }else{
                $disklimit=(int)$hostingdata['reseller_data']['reseller_quota'];
            }
        }else{
            $disklimit=0;
        }
    }else{
        if($hostingdata['bandwidth']=="unlimited"){
            $bwlimit=0;
        }else{
            $bwlimit=(int)$hostingdata['bandwidth'];
        }
        if($hostingdata['quota']=="unlimited"){
            $disklimit=0;
        }else{
            $disklimit=(int)$hostingdata['quota'];
        }
    }
    
    /*    
    if($hostingdata['bandwidth']=="unlimited"){
        $bwlimit=0;
    }else{
        $bwlimit=(int)$hostingdata['bandwidth'];
    }
    if($hostingdata['quota']=="unlimited"){
        $disklimit=0;
    }else{
        $disklimit=(int)$hostingdata['quota'];
    }
    */
    $created_at=$hostingdata['setuptime'];
    if(!empty($hostingdata['nameservers'][0])){
        $ns1=$hostingdata['nameservers'][0];
    }else{
        $ns1='';        
    }
    if(!empty($hostingdata['nameservers'][1])){
        $ns2=$hostingdata['nameservers'][1];
    }else{
        $ns2='';        
    }
    if ($producttype == "reselleraccount"){
        if(!empty($hostingdata['quota_usage_reseller'])){
            $diskusage=(int)$hostingdata['quota_usage_reseller'];
        }else{
            $diskusage=0;
        }
        if(!empty($hostingdata['bandwidth_usage_reseller'])){
            $bwusage=(int)$hostingdata['bandwidth_usage_reseller'];
        }else{
            $bwusage=0;
        }
    }else{
        if(!empty($hostingdata['quota_usage'])){
            $diskusage=(int)$hostingdata['quota_usage'];
        }else{
            $diskusage=0;
        }
        if(!empty($hostingdata['bandwidth_usage'])){
            $bwusage=(int)$hostingdata['bandwidth_usage'];
        }else{
            $bwusage=0;
        }
    }

    /*
    if(!empty($hostingdata['quota_usage'])){
        $diskusage=(int)$hostingdata['quota_usage'];
    }else{
        $diskusage=0;
    }
    if(!empty($hostingdata['bandwidth_usage'])){
        $bwusage=(int)$hostingdata['bandwidth_usage'];
    }else{
        $bwusage=0;
    }
    */
    Capsule::table('tblhosting')
        ->where('server', $params['serverid'])
        ->where('username',$user)               
        ->update([            
            'disklimit' => $disklimit,
            'ns1' => $ns1,
            'ns2' => $ns2,
            'bwlimit' => $bwlimit,
            'diskusage' => $diskusage,
            'bwusage' => $bwusage,
            'created_at' => $created_at,
            'lastupdate' => Capsule::raw('now()'),
        ]);    

}
/*
Daily account usage sync to whmcs
*/
function eenos_UsageUpdate(array $params)
{
    try {
        $server_id=$params['serverid'];
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";          
        $post_url='/usagestatistics/';             
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);           
        
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        if (!empty($result["message"]["users"])){
            try {
                $userssummary=$result["message"]["users"]; 
                $meta_data_json=Capsule::table("tblhosting")->where("server", $server_id)->select('username')->get();
                $usersarray=json_decode($meta_data_json,true); 
                foreach($usersarray as $key => $value){
                    $users[]=$value["username"];                
                } 
                foreach($users as $user){                
                    if(!empty($userssummary[$user])){                    
                        eenos_save_usage_data($params,$user,$userssummary[$user]);
                    }
                    // eenos_debug_log($userssummary[$user]);
                }         
            }catch (Exception $e) {
                // Record the error in WHMCS's module log.
                logModuleCall(
                    'eenos',
                    __FUNCTION__,
                    $params,
                    $e->getMessage(),
                    $e->getTraceAsString()
                );        
                return $e->getMessage();
            }
        }                
    }
}

function eenos_ListAccounts(array $params)
{
    try {
        $server_id=$params['serverid'];
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";          
        $post_url='/listaccts/';             
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);   
       
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    $accounts=[];
    if($result['success']=="200"){                  
        eenos_debug_log($result["data"]["data"]["accounts"]);
        $userdata=$result["data"]["data"]["accounts"];
        $status_active= WHMCS\Service\Status::ACTIVE;
        $status_suspended=WHMCS\Service\Status::SUSPENDED;
        foreach($userdata as $user => $udata){
            $createdDate = NULL;
            $startDate =$udata['setuptime'];            
            $startDateObject = WHMCS\Carbon::createFromTimeString($startDate);
            if($startDateObject){
                $createdDate = $startDateObject->toDateTimeString();
            }            
            if($udata['is_suspended'] == "no"){
                $status=$status_active;
            }else{
                $status=$status_suspended;
            }
            $accounts[]=array(
                'name' => $user,
                'email' => $udata['contactemail'],
                'username' =>$user,
                'domain' => $udata['domain'],
                'uniqueIdentifier' => $udata['domain'],
                'product' => $udata['package'],
                'primaryip' => $udata['dedicatedip']['ipv4'],
                'created' => $createdDate,
                'status' => $status
            );  
        }
        eenos_debug_log($accounts);
        return array('success' => true, 'accounts' => $accounts);        
    }else{
        return array('success' => false, 'accounts' => $accounts, 'error' => $result['message']);
    }
}

/**
 * Additional actions an admin user can invoke.
 *
 * Define additional actions that an admin user can perform for an
 * instance of a product/service.
 *
 * @see eenos_buttonOneFunction()
 *
 * @return array
 */

 function eenos_AdminCustomButtonArray()
 {
     return array(
         "Sync Data" => "buttonSyncDataFunction",        
     );
 }
 /*
  Custom Button Sync Data Functio.
  This will update the whmcs with the data from the server
  
 */

function eenos_buttonSyncDataFunction(array $params)
{
    try {
        $apidata['apiuser']=$params["serverusername"];
        $apidata['apiauth']=$params["serverpassword"];
        $apidata['apiowner']="root";
        $apidata['user']=$params["username"];   
        $post_url='/accountsummary/';             
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);                   
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
        $result=eenos_PerformOperation($apidata,$action_url);
        eenos_debug_log($apidata);        
        eenos_debug_log($result);           
        
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        if (!empty($result["message"]["acct"])){
            $summary=$result["message"]["acct"];            
            eenos_debug_log($summary);
            eenos_save_user_data($params,$summary,True);
        }        
        return 'success';
    }else{
        return($result['message']);
    }
    
}


/*
 * Additional actions a client user can invoke.
 * 
*/
function eenos_ClientAreaCustomButtonArray()
{
    return array(
        "Login to Eenos" => "ClientSignleSignOn",
        // "Action 1 Display Value" => "actionOneFunction",
        // "Action 2 Display Value" => "actionTwoFunction",
    );
}
function eenos_ClientSignleSignOn(array $params)
{
    try {
        // eenos_debug_log($params);
        $forceuser=trim(isset($_REQUEST['u']) ? $_REQUEST['u'] : '');
        // eenos_debug_log($forceuser);
        $proto=$params["serverhttpprefix"];
        $port=$params["serverport"];
        $server=get_eenos_serverip($params);
        if( $params["producttype"] == "reselleraccount"){
            if (!empty($forceuser) && $forceuser == "ok"){
                $post_url='/uappsso/';
            }else{
                $post_url='/rappsso/';
            }
        }else{
            $post_url='/uappsso/';
        }
        $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;
        $post_data['apiuser']=$params["serverusername"];
        $post_data['apiauth']=$params["serverpassword"];
        $post_data['user']= $params["username"];
        // eenos_debug_log( $post_data) ;
        // eenos_debug_log($action_url);
        $result=eenos_PerformOperation($post_data,$action_url);
        eenos_debug_log($result);
        $response = array();
        if(!empty($result['data']['data']['url'])){
            $success = true;
            $response['redirectUrl']=$result['data']['data']['url'];
            $errorMsg = '';
            eenos_debug_log($response);
            echo '<meta http-equiv="Refresh" content="0;url='.$response['redirectUrl'].'">';
            exit;
            // return array(
            //     'success' => true,               
            //     'redirectTo' => $response['redirectUrl'],
            // );
        }else{
            $success = false;
            $response['redirectUrl']='#';
            $errorMsg = $result['message'];
            return array(
                'success' => false,
                'errorMsg' => $errorMsg,
            );
        }              
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}
/**
 * Custom function for performing an additional action.
 *
 * You can define an unlimited number of custom functions in this way.
 *
 * Similar to all other module call functions, they should either return
 * 'success' or an error message to be displayed.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see eenos_AdminCustomButtonArray()
 *
 * @return string "success" or an error message
 */
// function eenos_buttonOneFunction(array $params)
// {
//     try {
//         // Call the service's function, using the values provided by WHMCS in
//         // `$params`.
//     } catch (Exception $e) {
//         // Record the error in WHMCS's module log.
//         logModuleCall(
//             'eenos',
//             __FUNCTION__,
//             $params,
//             $e->getMessage(),
//             $e->getTraceAsString()
//         );

//         return $e->getMessage();
//     }

//     return 'success';
// }

/**
 * Custom function for performing an additional action.
 *
 * You can define an unlimited number of custom functions in this way.
 *
 * Similar to all other module call functions, they should either return
 * 'success' or an error message to be displayed.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see eenos_ClientAreaCustomButtonArray()
 *
 * @return string "success" or an error message
 */
// function eenos_actionOneFunction(array $params)
// {
//     try {
//         // Call the service's function, using the values provided by WHMCS in
//         // `$params`.
//     } catch (Exception $e) {
//         // Record the error in WHMCS's module log.
//         logModuleCall(
//             'eenos',
//             __FUNCTION__,
//             $params,
//             $e->getMessage(),
//             $e->getTraceAsString()
//         );

//         return $e->getMessage();
//     }

//     return 'success';
// }

/**
 * Admin services tab additional fields.
 *
 * Define additional rows and fields to be displayed in the admin area service
 * information and management page within the clients profile.
 *
 * Supports an unlimited number of additional field labels and content of any
 * type to output.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see eenos_AdminServicesTabFieldsSave()
 *
 * @return array
 */
function eenos_AdminServicesTabFields(array $params)
{
    try {        
        
        // eenos_debug_log($params["pid"]);
        $pid=(int)$params["pid"];
        $plan=$params['configoption1'];
        $meta_data_json=Capsule::table("tblhosting")->where("id", $pid)->get();
        $hdata=json_decode($meta_data_json,true)[0];
        // eenos_debug_log($meta_data_json);
        // eenos_debug_log($hdata);
        $ipv4=$ipv6='';        
        if (!empty($hdata["assignedips"])){
            $ips=explode(',',$hdata["assignedips"]);
            foreach($ips as $ip){
                if (preg_match('/:/i', $ip)) {
                    $ipv6=$ip;
                }else{
                    $ipv4=$ip; 
                }                
            }
        }
        $ns1=$ns2='';
        if(!empty($hdata["ns1"])){
            $ns1= $hdata["ns1"];
        }
        if(!empty($hdata["ns2"])){
            $ns2= $hdata["ns2"];
        }
        
        /*
        $server_id=Capsule::table("tblservers")->where("ipaddress", $params["serverip"])->value("id");
        $meta_data_json=Capsule::table("tblservers_remote")->where("server_id", $server_id)->value("meta_data");
        $meta_data=json_decode($meta_data_json,true);
        if (!empty($meta_data['packages'])){
            $packages=$meta_data['packages'];
        }else{
            $packages="Default";
        }
        $pkgs=explode(",",$packages);
        sort($pkgs);       
        $pdata='';
        foreach($pkgs as $pname){
            if($pname == $plan){
                $pdata.="<option value=\"$pname\" selected >$pname</option>\n";
            }else{
                $pdata.="<option value=\"$pname\">$pname</option>\n";
            }
        }
        $fpkgs="<select name=\"newpackage\" class=\"form-control select-inline\">\n$pdata\n</select>";
        eenos_debug_log($server_id);
        eenos_debug_log($meta_data);        
        */
        $response = array();

        // Return an array based on the function's response.
        return array(
            'Hosting Package' => $plan,
            'Hosting IP(v4)'=> $ipv4,  
            'Hosting IP(v6)'=> $ipv6,
            'Name Server 1'=> $ns1,
            'Name Server 2'=> $ns2,
            // 'Change Hosting Package' => $fpkgs.'&nbsp;<button type="submit" class="btn btn-sm btn-primary  btn-outline-primary" id="cbtnButton_package" name="ChangeNewPlan" value="yes">Change Plan</button>',            
            
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        // In an error condition, simply return no additional fields to display.
    }

    return array();
}

/**
 * Execute actions upon save of an instance of a product/service.
 *
 * Use to perform any required actions upon the submission of the admin area
 * product management form.
 *
 * It can also be used in conjunction with the AdminServicesTabFields function
 * to handle values submitted in any custom fields which is demonstrated here.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see eenos_AdminServicesTabFields()
 */
function eenos_AdminServicesTabFieldsSave(array $params)
{
    $cplan=$params['configoption1'];
    $newplan=trim(isset($_REQUEST['newpackage']) ? $_REQUEST['newpackage'] : '');
    $dochangenewplan=trim(isset($_REQUEST['ChangeNewPlan']) ? $_REQUEST['ChangeNewPlan'] : '');
    eenos_debug_log($newplan);
    eenos_debug_log($dochangenewplan);
    if ((!empty($newplan) && !empty($dochangenewplan)) && ($newplan != $cplan )){
        eenos_debug_log("Changing hosting plan manually on server from ".$cplan." to ".$newplan);        
        try {
            $apidata['apiuser']=$params["serverusername"];
            $apidata['apiauth']=$params["serverpassword"];
            $apidata['apiowner']="root";
            $apidata['user']=$params["username"]; 
            $apidata['package']=$newplan;                   
            $post_url='/changepackage/';          
            $proto=$params["serverhttpprefix"];
            $port=$params["serverport"];
            $server=get_eenos_serverip($params);                   
            $action_url=$proto.'://'.$server.':'.$port.'/api'.$post_url;        
            $result=eenos_PerformOperation($apidata,$action_url);             
            eenos_debug_log($result);     
            // Call the service's function, using the values provided by WHMCS
            // in `$params`.
        } catch (Exception $e) {
            // Record the error in WHMCS's module log.
            logModuleCall(
                'eenos',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
            );

            // Otherwise, error conditions are not supported in this operation.
        }
        if($result['success']=="200"){
            
            return 'success';
        }else{
            return($result['message']);
        }
         
    }

    // Fetch form submission variables.
    $originalFieldValue = isset($_REQUEST['eenos_original_uniquefieldname'])
        ? $_REQUEST['eenos_original_uniquefieldname']
        : '';

    $newFieldValue = isset($_REQUEST['eenos_uniquefieldname'])
        ? $_REQUEST['eenos_uniquefieldname']
        : '';
    $newplan=trim(isset($_REQUEST['newpackage']) ? $_REQUEST['newpackage'] : '');
    
    // Look for a change in value to avoid making unnecessary service calls.
    if ($originalFieldValue != $newFieldValue) {
        try {
            // Call the service's function, using the values provided by WHMCS
            // in `$params`.
        } catch (Exception $e) {
            // Record the error in WHMCS's module log.
            logModuleCall(
                'eenos',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
            );

            // Otherwise, error conditions are not supported in this operation.
        }
    }
}

/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns a URL to which the user should be redirected.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */



/**
 * Client area output logic handling.
 *
 * This function is used to define module specific client area output. It should
 * return an array consisting of a template file and optional additional
 * template variables to make available to that template.
 *
 * The template file you return can be one of two types:
 *
 * * tabOverviewModuleOutputTemplate - The output of the template provided here
 *   will be displayed as part of the default product/service client area
 *   product overview page.
 *
 * * tabOverviewReplacementTemplate - Alternatively using this option allows you
 *   to entirely take control of the product/service overview page within the
 *   client area.
 *
 * Whichever option you choose, extra template variables are defined in the same
 * way. This demonstrates the use of the full replacement.
 *
 * Please Note: Using tabOverviewReplacementTemplate means you should display
 * the standard information such as pricing and billing details in your custom
 * template or they will not be visible to the end user.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function eenos_ClientArea(array $params)
{
    try {

        $ProductId = 'abc';
        $extraVariable2 = '123';

        return array(
            
            'templateVariables' => array(
                'extraVariable1' => $extraVariable1,
                'extraVariable2' => $extraVariable2,
            ),
        );
    } catch (Exception $e) {
              // Record the error in WHMCS's module log.
        logModuleCall(
            'eenos',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
    }  
}
// function eenos_ClientArea(array $params)
// {
//     // Determine the requested action and set service call parameters based on
//     // the action.
//     $requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : '';

//     if ($requestedAction == 'manage') {
//         $serviceAction = 'get_usage';
//         $templateFile = 'templates/manage.tpl';
//     } else {
//         $serviceAction = 'get_stats';
//         $templateFile = 'templates/overview.tpl';
//     }

//     try {
//         // Call the service's function based on the request action, using the
//         // values provided by WHMCS in `$params`.
//         $response = array();

//         $extraVariable1 = 'abc';
//         $extraVariable2 = '123';

//         return array(
//             'tabOverviewReplacementTemplate' => $templateFile,
//             'templateVariables' => array(
//                 'extraVariable1' => $extraVariable1,
//                 'extraVariable2' => $extraVariable2,
//             ),
//         );
//     } catch (Exception $e) {
//         // Record the error in WHMCS's module log.
//         logModuleCall(
//             'eenos',
//             __FUNCTION__,
//             $params,
//             $e->getMessage(),
//             $e->getTraceAsString()
//         );

//         // In an error condition, display an error page.
//         return array(
//             'tabOverviewReplacementTemplate' => 'error.tpl',
//             'templateVariables' => array(
//                 'usefulErrorHelper' => $e->getMessage(),
//             ),
//         );
//     }
// }



function eenos_server_pass_from_whmcs($enc_pass){
		
	global $cc_encryption_hash;
	
	// Include WHMCS database configuration file
	include_once(dirname(dirname(dirname(dirname(__FILE__)))).'/configuration.php');
	$key1 = md5 (md5 ($cc_encryption_hash));
	$key2 = md5 ($cc_encryption_hash);
	$key = $key1.$key2;
	$hasher = new eenos_hash_encryption($key);
	return $hasher->decrypt($enc_pass);
}

// WHMCS Decrypter
class eenos_hash_encryption {
	/**
	 * Hashed value of the user provided encryption key
	 * @var	string
	 **/
	var $hash_key;
	/**
	 * String length of hashed values using the current algorithm
	 * @var	int
	 **/	
	var $hash_lenth;
	/**
	 * Switch base64 enconding on / off
	 * @var	bool	true = use base64, false = binary output / input
	 **/	
	var $base64;
	/**
	 * Secret value added to randomize output and protect the user provided key
	 * @var	string	Change this value to add more randomness to your encryption
	 **/	
	var $salt = 'Change this to any secret value you like. "d41d8cd98f00b204e9800998ecf8427e" might be a good example.';
	

	/**
	 * Constructor method
	 *
	 * Used to set key for encryption and decryption.
	 * @param	string	$key	Your secret key used for encryption and decryption
	 * @param	boold	$base64	Enable base64 en- / decoding
	 * @return mixed
	 */
	function __construct($key, $base64 = true) {
		
		global $cc_encryption_hash;
		
		// Toggle base64 usage on / off
		$this->base64 = $base64;
		
		// Instead of using the key directly we compress it using a hash function
		$this->hash_key = $this->_hash($key);
		
		// Remember length of hashvalues for later use
		$this->hash_length = strlen($this->hash_key);
	}
		
	/**
	 * Method used for encryption
	 * @param	string	$string	Message to be encrypted
	 * @return string	Encrypted message
	 */
	function encrypt($string) {
		$iv = $this->_generate_iv();
		
		// Clear output
		$out = '';
		
		// First block of output is ($this->hash_hey XOR IV)
		for($c=0;$c < $this->hash_length;$c++) {
			$out .= chr(ord($iv[$c]) ^ ord($this->hash_key[$c]));
		}

		// Use IV as first key
		$key = $iv;
		$c = 0;

		// Go through input string
		while($c < strlen($string)) {
			// If we have used all characters of the current key we switch to a new one
			if(($c != 0) and ($c % $this->hash_length == 0)) {
				// New key is the hash of current key and last block of plaintext
				$key = $this->_hash($key . substr($string,$c - $this->hash_length,$this->hash_length));
			}
			// Generate output by xor-ing input and key character for character
			$out .= chr(ord($key[$c % $this->hash_length]) ^ ord($string[$c]));
			$c++;
		}
		// Apply base64 encoding if necessary
		if($this->base64) $out = base64_encode($out);
		return $out;
	}
	
	/**
	 * Method used for decryption
	 * @param	string	$string	Message to be decrypted
	 * @return string	Decrypted message
	 */
	function decrypt($string) {
		// Apply base64 decoding if necessary
		if($this->base64) $string = base64_decode($string);
		
		// Extract encrypted IV from input
		$tmp_iv = substr($string,0,$this->hash_length);
		
		// Extract encrypted message from input
		$string = substr($string,$this->hash_length,strlen($string) - $this->hash_length);
		$iv = $out = '';
		
		// Regenerate IV by xor-ing encrypted IV from block 1 and $this->hashed_key
		// Mathematics: (IV XOR KeY) XOR Key = IV
		for($c=0;$c < $this->hash_length;$c++) 
		{
			$iv .= chr(ord($tmp_iv[$c]) ^ ord($this->hash_key[$c]));
		}
		// Use IV as key for decrypting the first block cyphertext
		$key = $iv;
		$c = 0;
		
		// Loop through the whole input string
		while($c < strlen($string)) {
			// If we have used all characters of the current key we switch to a new one
			if(($c != 0) and ($c % $this->hash_length == 0)) {
				// New key is the hash of current key and last block of plaintext
				$key = $this->_hash($key . substr($out,$c - $this->hash_length,$this->hash_length));
			}
			// Generate output by xor-ing input and key character for character
			$out .= chr(ord($key[$c % $this->hash_length]) ^ ord($string[$c]));
			$c++;
		}
		return $out;
	}

	/**
	 * Hashfunction used for encryption
	 *
	 * This class hashes any given string using the best available hash algorithm.
	 * Currently support for md5 and sha1 is provided. In theory even crc32 could be used
	 * but I don't recommend this.
	 *
	 * @access	private
	 * @param	string	$string	Message to hashed
	 * @return string	Hash value of input message
	 */
	function _hash($string) {
		// Use sha1() if possible, php versions >= 4.3.0 and 5
		if(function_exists('sha1')) {
			$hash = sha1($string);
		} else {
			// Fall back to md5(), php versions 3, 4, 5
			$hash = md5($string);
		}
		$out ='';
		// Convert hexadecimal hash value to binary string
		for($c=0;$c<strlen($hash);$c+=2) {
			$out .= $this->_hex2chr($hash[$c] . $hash[$c+1]);
		}
		return $out;
	}
	
	/**
	 * Generate a random string to initialize encryption
	 *
	 * This method will return a random binary string IV ( = initialization vector).
	 * The randomness of this string is one of the crucial points of this algorithm as it
	 * is the basis of encryption. The encrypted IV will be added to the encrypted message
	 * to make decryption possible. The transmitted IV will be encoded using the user provided key.
	 *
	 * @todo	Add more random sources.
	 * @access	private
	 * @see function	hash_encryption
	 * @return string	Binary pseudo random string
	 **/
	function _generate_iv() {
		// Initialize pseudo random generator
		srand ((double)microtime()*1000000);
		
		// Collect random data.
		// Add as many "pseudo" random sources as you can find.
		// Possible sources: Memory usage, diskusage, file and directory content...
		$iv  = $this->salt;
		$iv .= rand(0,getrandmax());
		// Changed to serialize as the second parameter to print_r is not available in php prior to version 4.4
		$iv .= serialize($GLOBALS);
		return $this->_hash($iv);
	}
	
	/**
	 * Convert hexadecimal value to a binary string
	 *
	 * This method converts any given hexadecimal number between 00 and ff to the corresponding ASCII char
	 *
	 * @access	private
	 * @param	string	Hexadecimal number between 00 and ff
	 * @return	string	Character representation of input value
	 **/
	function _hex2chr($num) {
		return chr(hexdec($num));
	}
}